package cz.muni.fi.pb162.exception;

/**
 * Výnimka je problém, ktorý vznikne počas vykonávania programu
 * Pri jej vzniku je prerušený štandardný beh programu
 * Výnimky sú spôsob ako písať robustný a spoľahlivý kód, odolný voči chybám používateľa i programu samotného
 * Výnimku vyhodíme pomocou kľúčového slova throw
 * Výnimka môže mať jeden z troch pôvodov:
 * - chyba programátora
 * - chyba používateľa
 * - chyba hardware
 * Výnimky delíme na tri základné typy:
 * - error - ukázané v tomto balíku
 * - sledované výnimky - ukázané v tomto balíku
 * - behové výnimky - ukážky nájdete v balíku runtimeexception
 */
public class Demo {
    public static void main(String[] args) throws Exception {

        // Error sú chyby spôsobené prostredím
        // Napríklad nedostatočné množstvo voľnej operačnej pamäte
        // Je takmer nemožné aby sa z nich program zotavil
        // po odkomentovaní nasledujúceho riadku, program skončí s výnimkou OutOfMemoryError
        // outOfMemoryErrorSimulation();

        // Sledované výnimky reprezentujú chybový stav
        // Sú potomkami triedy Exception
        // Sú sledované prekladačom
        // Príklad vlastnej sledovanej výnimky je trieda CarNotStartedException
        // Príklad použitia nájdete v triede Car v metóde drive()
        // Príklad zachytenia nájdete v metóde driveNotStartedCar()
        Car audi = new Car("Audi");
        Car bmw = new Car("BMW");
        driveCar(audi);
        driveCar(bmw);


    }

    // táto metóda simuluje prečerpanie povolenej pamäte alokáciou stále väčšieho poľa
    private static void outOfMemoryErrorSimulation() {
        int dummyArraySize = 15;
        long[] memoryAllocated = null;
        for (int loop = 0; loop < Integer.MAX_VALUE; loop++) {
            memoryAllocated = new long[dummyArraySize];
            memoryAllocated[0] = 0;
            dummyArraySize *= dummyArraySize * 2;
        }
    }

    private static void driveCar(Car car) {
        do {
            try {
                System.out.println("Snažím sa jazdiť s naštartovaným " + car);

                //car = null;
                car.drive();

                System.out.println("S autami vieme jazdiť, keďže bolo naštartované!");

            } catch (CarNotStartedException ex) {

                Car errorCar = ex.getErrorCar();

                System.out.println("Auto nenastartované = " + errorCar);

                errorCar.startCar();
                errorCar.drive();

                System.out.println("Auto už nastartované = " + errorCar);

            } catch (Exception ex) {
                // sem jdu pri jakekoli jine vyjimce
                ex.printStackTrace();

            } finally {
                System.out.println("Always here for " + car);
            }
        } while (false);
        // here the bmw is started
    }
}
